#pragma once
#include <string>
#include <vector>
#include <functional>

static double from_range_to_range(double v, double input_min, double input_max, double output_min, double output_max)
{
    double range1=input_max-input_min;
    double range2=output_max-output_min;
    return output_min+(range2/range1)*(v-input_min);
}

class hourglass_plugin_parameter
{
public:
    hourglass_plugin_parameter(std::string name,
                            std::string unit_name,
                            double min_value,
                            double max_value,
                            double default_value,
                            std::function<void(double)> f_post_set=[](double){}) :

        m_name(name),
        m_unit_name(unit_name),
        m_min_val(min_value),
        m_max_val(max_value),
        post_set(f_post_set)
    {
        to_normalized=[min_value,max_value](double x){ return from_range_to_range(x,min_value,max_value,0.0,1.0); };
        from_normalized=[min_value,max_value](double x){ return from_range_to_range(x,0.0,1.0,min_value,max_value); };
        set_normalized_value(to_normalized(default_value));
    }
    std::string m_name;
    std::string m_unit_name;
    std::function<double(double)> from_normalized;
    std::function<double(double)> to_normalized;
    std::function<void(double)> post_set;
    double get_value() { return from_normalized(m_value_normalized); }
    void set_normalized_value(double x)
    {
        m_value_normalized=x;
        post_set(from_normalized(x));
    }
    double get_normalized() const { return m_value_normalized; }
private:
    double m_value_normalized;
    double m_min_val;
    double m_max_val;
};

struct pitch_info
{
    double hz;
    double qual;
};

class IHourGlass_pitch_tracker
{
public:
    virtual ~IHourGlass_pitch_tracker() {}
    virtual int num_parameters()=0;
    virtual hourglass_plugin_parameter* get_parameter(size_t index)=0;
    virtual void reset()=0;
    virtual bool process(double* audio_in, size_t frames, double sr, std::vector<pitch_info>& output)=0;
    virtual std::string name()=0;
};
