#ifndef ZOLZER_PITCH_TRACK_H
#define ZOLZER_PITCH_TRACK_H

/*
  PLL Pitch tracker (Zoelzer et al)
   V Lazzarini, 2012

  C++11'ied and build as dll, Xenakios, 2014
  
  This file has parts from Csound.

  The Csound Library is free software; you can redistribute it
  and/or modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Csound is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with Csound; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  02111-1307 USA
*/

#include "Windows.h"
#include "hourglass_plugin.h"
#include <vector>
#include <memory>
#include <string>
#include <iostream>

#define DLL_EXPORT __declspec(dllexport) 

typedef double MYFLT;

// simplified "faked" CSOUND structure to avoid having to include csound headers

struct pitch_track_CSOUND
{
    pitch_track_CSOUND() : pidsr(3.141592654/44100.0), e0dbfs(1.0), ksmps(0), sr(44100.0) {}
    double pidsr;
    double e0dbfs;
    int ksmps;
    int GetKsmps() const { return ksmps; }
    double GetSr() const { return sr; }
    double sr;
};

const double ROOT2=1.4142135623730950488;

enum {LP1=0, LP2, HP};

struct BIQUAD
{
    double a0, a1, a2, b1, b2;
    double del1, del2;
};

struct PLLTRACK
{
    PLLTRACK() : kd(0.5),
        freq(4096), lock(4096), klpf(20.0), klpfQ(0.333),
        klf(20.0), khf(1500.0), kthresh(0.001) {}
    std::vector<MYFLT> freq;
    std::vector<MYFLT> lock;
    MYFLT kd;
    MYFLT klpf;
    MYFLT klpfQ;
    MYFLT klf;
    MYFLT khf;
    MYFLT kthresh;
    MYFLT *asig;
    BIQUAD   fils[6];
    double  ace, xce;
    double cos_x, sin_x, x1, x2;
    MYFLT klpf_o, klpfQ_o, klf_o,khf_o;

};

void update_coefs(pitch_track_CSOUND *csound, double fr, double Q, BIQUAD *biquad, int TYPE);

int plltrack_set(pitch_track_CSOUND *csound, PLLTRACK *p);

int plltrack_perf(pitch_track_CSOUND *csound, PLLTRACK *p);

class pll_pitch_tracker : public IHourGlass_pitch_tracker
{
public:
    pll_pitch_tracker();
    std::string name() { return "PLL Pitch Tracker 1.0"; }
    int num_parameters() { return m_params.size(); }
    hourglass_plugin_parameter* get_parameter(size_t index)
    {
        if (index<m_params.size())
            return &m_params[index];
        return nullptr;
    }
    void reset() { plltrack_set(&csound, &pll); }
    bool process(double* audio_in, size_t frames, double sr,std::vector<pitch_info>& output);
private:
    pitch_track_CSOUND csound;
    PLLTRACK pll;
    std::vector<hourglass_plugin_parameter> m_params;
};

extern "C"
{
DLL_EXPORT IHourGlass_pitch_tracker* create_pitch_tracker();
DLL_EXPORT void destroy_pitch_tracker(IHourGlass_pitch_tracker*);
}

#endif // ZOLZER_PITCH_TRACK_H
